import os
import sys
import warnings
from importlib import import_module
from optparse import OptionParser, make_option

import django
from django.apps import apps
from django.conf import settings
from django.core.management.commands.test import Command as TestCommand

from django_jenkins.runner import CITestSuiteRunner


def get_runner(settings, test_runner_class=None):
    if test_runner_class is None:
        test_runner_class = getattr(
            settings, 'JENKINS_TEST_RUNNER', 'django_jenkins.runner.CITestSuiteRunner')

    test_module_name, test_cls = test_runner_class.rsplit('.', 1)
    test_module = __import__(test_module_name, {}, {}, test_cls)
    test_runner = getattr(test_module, test_cls)

    if not issubclass(test_runner, CITestSuiteRunner):
        raise ValueError('Your custom TestRunner should extend '
                         'the CITestSuiteRunner class.')
    return test_runner


class Command(TestCommand):
    # TODO Remove, when drop django 1.7 support
    option_list = TestCommand.option_list + (
        make_option('--output-dir', dest='output_dir', default="reports",
                    help='Report files directory'),
        make_option("--enable-coverage",
                    action="store_true", default=False,
                    help="Measure code coverage"),
        make_option('--debug', action='store_true',
                    dest='debug', default=False,
                    help='Do not intercept stdout and stderr, friendly for console debuggers'),
        make_option("--coverage-rcfile",
                    dest="coverage_rcfile",
                    default="",
                    help="Specify configuration file."),
        make_option("--coverage-format",
                    dest="coverage_format",
                    default="xml",
                    help="Specify coverage output formats html,xml,bin"),
        make_option("--coverage-exclude", action="append",
                    default=[], dest="coverage_excludes",
                    help="Module name to exclude"),
        make_option("--project-apps-tests", action="store_true",
                    default=False, dest="project_apps_tests",
                    help="Take tests only from project apps")
    )

    def __init__(self):
        super(Command, self).__init__()
        self.tasks_cls = [import_module(module_name).Reporter
                          for module_name in self.get_task_list()]
        self.tasks = [task_cls() for task_cls in self.tasks_cls]

    def get_task_list(self):
        tasks = getattr(settings, 'JENKINS_TASKS', ())
        if '--enable-coverage' in sys.argv and 'django_jenkins.tasks.with_coverage' not in tasks:
            try:
                from django.apps import apps  # NOQA
            except ImportError:
                """
                We are on django 1.6
                """
                tasks += ('django_jenkins.tasks.with_coverage',)
        return tasks

    @property
    def use_argparse(self):
        return True

    def add_arguments(self, parser):
        super(Command, self).add_arguments(parser)
        parser.add_argument('--output-dir', dest='output_dir', default="reports",
                            help='Report files directory'),
        parser.add_argument("--enable-coverage",
                            action="store_true", default=False,
                            help="Measure code coverage"),
        parser.add_argument('--debug', action='store_true',
                            dest='debug', default=False,
                            help='Do not intercept stdout and stderr, friendly for console debuggers'),
        parser.add_argument("--coverage-rcfile",
                            dest="coverage_rcfile",
                            default="",
                            help="Specify configuration file."),
        parser.add_argument("--coverage-format",
                            dest="coverage_format",
                            default="xml",
                            help="Specify coverage output formats html,xml,bin"),
        parser.add_argument("--coverage-exclude", action="append",
                            default=[], dest="coverage_excludes",
                            help="Module name to exclude"),
        parser.add_argument("--project-apps-tests", action="store_true",
                            default=False, dest="project_apps_tests",
                            help="Take tests only from project apps")

        parser._optionals.conflict_handler = 'resolve'
        for task in self.tasks:
            if hasattr(task, 'add_arguments'):
                task.add_arguments(parser)

    def create_parser(self, prog_name, subcommand):
        if django.VERSION >= (1, 8):
            parser = super(Command, self).create_parser(prog_name, subcommand)
        else:
            # TODO Remove, when drop django 1.7 support
            test_runner_class = get_runner(settings, self.test_runner)
            options = self.option_list + getattr(
                test_runner_class, 'option_list', ())

            for task in self.tasks:
                options += tuple(option for option in getattr(task, 'option_list', ())
                                 if all(option._long_opts[0] != existing._long_opts[0]
                                        for existing in options))

            parser = OptionParser(prog=prog_name,
                                  usage=self.usage(subcommand),
                                  version=self.get_version(),
                                  option_list=options)

        return parser

    def handle(self, *test_labels, **options):
        TestRunner = get_runner(settings, options.get('testrunner'))
        options['verbosity'] = int(options.get('verbosity'))

        if options.get('liveserver') is not None:
            os.environ['DJANGO_LIVE_TEST_SERVER_ADDRESS'] = options['liveserver']
            del options['liveserver']

        output_dir = options['output_dir']
        if not os.path.exists(output_dir):
            os.makedirs(output_dir)

        test_runner = TestRunner(**options)

        if not test_labels and options['project_apps_tests']:
            test_labels = getattr(settings, 'PROJECT_APPS', [])

        failures = test_runner.run_tests(test_labels)

        if failures:
            sys.exit(bool(failures))
        else:
            tested_locations = self.get_tested_locations(test_labels)

            coverage = apps.get_app_config('django_jenkins').coverage
            if coverage:
                if options['verbosity'] >= 1:
                    print('Storing coverage info...')

                coverage.save(tested_locations, options)

            # run reporters
            for task in self.tasks:
                if options['verbosity'] >= 1:
                    print('Executing {0}...'.format(task.__module__))
                task.run(tested_locations, **options)

            if options['verbosity'] >= 1:
                print('Done')

    def get_tested_locations(self, test_labels):
        locations = []

        if test_labels:
            pass
        elif hasattr(settings, 'PROJECT_APPS'):
            test_labels = settings.PROJECT_APPS
        else:
            warnings.warn('No PROJECT_APPS settings, coverage gathered over all apps')
            test_labels = settings.INSTALLED_APPS

        for test_label in test_labels:
            app_config = apps.get_containing_app_config(test_label)
            if app_config is not None:
                locations.append(os.path.dirname(app_config.module.__file__))
            else:
                warnings.warn('No app found for test: {0}'.format(test_label))

        return locations



import os.path
import pep8
from optparse import make_option
from django.conf import settings

from . import set_option


class Reporter(object):
    # TODO Remove, when drop django 1.7 support
    option_list = (
        make_option("--pep8-exclude",
                    dest="pep8-exclude",
                    help="exclude files or directories which match these "
                    "comma separated patterns (default: %s)" %
                    (pep8.DEFAULT_EXCLUDE + ",south_migrations")),
        make_option("--pep8-select", dest="pep8-select",
                    help="select errors and warnings (e.g. E,W6)"),
        make_option("--pep8-ignore", dest="pep8-ignore",
                    help="skip errors and warnings (e.g. E4,W)"),
        make_option("--pep8-max-line-length",
                    dest="pep8-max-line-length", type='int',
                    help="set maximum allowed line length (default: %d)" %
                    pep8.MAX_LINE_LENGTH),
        make_option("--pep8-rcfile", dest="pep8-rcfile",
                    help="PEP8 configuration file")
    )

    def add_arguments(self, parser):
        parser.add_argument("--pep8-exclude",
                            dest="pep8-exclude",
                            help="exclude files or directories which match these "
                            "comma separated patterns (default: %s)" %
                            (pep8.DEFAULT_EXCLUDE + ",south_migrations"))
        parser.add_argument("--pep8-select", dest="pep8-select",
                            help="select errors and warnings (e.g. E,W6)")
        parser.add_argument("--pep8-ignore", dest="pep8-ignore",
                            help="skip errors and warnings (e.g. E4,W)"),
        parser.add_argument("--pep8-max-line-length",
                            dest="pep8-max-line-length", type=int,
                            help="set maximum allowed line length (default: %d)" % pep8.MAX_LINE_LENGTH)
        parser.add_argument("--pep8-rcfile", dest="pep8-rcfile",
                            help="PEP8 configuration file")

    def run(self, apps_locations, **options):
        output = open(os.path.join(options['output_dir'], 'pep8.report'), 'w')

        class JenkinsReport(pep8.BaseReport):
            def error(instance, line_number, offset, text, check):
                code = super(JenkinsReport, instance).error(line_number, offset, text, check)
                if code:
                    sourceline = instance.line_offset + line_number
                    output.write('%s:%s:%s: %s\n' % (instance.filename, sourceline, offset + 1, text))

        pep8_options = {}
        config_file = self.get_config_path(options)
        if config_file is not None:
            pep8_options['config_file'] = config_file

        set_option(pep8_options, 'exclude', options['pep8-exclude'], config_file,
                   default=pep8.DEFAULT_EXCLUDE + ",south_migrations", split=',')

        set_option(pep8_options, 'select', options['pep8-select'], config_file, split=',')

        set_option(pep8_options, 'ignore', options['pep8-ignore'], config_file, split=',')

        set_option(pep8_options, 'max_line_length', options['pep8-max-line-length'], config_file,
                   default=pep8.MAX_LINE_LENGTH)

        pep8style = pep8.StyleGuide(
            parse_argv=False,
            reporter=JenkinsReport,
            **pep8_options)

        pep8style.options.report.start()
        for location in apps_locations:
            pep8style.input_dir(os.path.relpath(location))
        pep8style.options.report.stop()

        output.close()

    def get_config_path(self, options):
        if options['pep8-rcfile']:
            return options['pep8-rcfile']

        rcfile = getattr(settings, 'PEP8_RCFILE', None)
        if rcfile:
            return rcfile

        if os.path.exists('tox.ini'):
            return 'tox.ini'

        if os.path.exists('setup.cfg'):
            return 'setup.cfg'



import os
import sys
from importlib import import_module

from django.conf import settings


class CoverageReporter(object):
    def __init__(self):
        try:
            import coverage
            if coverage.__version__ < '4':
                raise ImportError('coverage>=4 required')
        except ImportError:
            raise ImportError('coverage is not installed')
        else:
            coverage_config_file = None
            for argv in sys.argv:
                if argv.startswith('--coverage-rcfile='):
                    _, coverage_config_file = argv.split('=')

            self.coverage = coverage.coverage(
                branch=True,
                config_file=coverage_config_file or self.default_coverage_config())
            self.coverage.start()

    def save(self, apps_locations, options):
        self.coverage.stop()
        self.coverage.get_data()
        morfs = self.get_morfs(self.coverage, apps_locations, options)

        if 'xml' in options['coverage_format']:
            self.coverage.xml_report(morfs=morfs, outfile=os.path.join(options['output_dir'], 'coverage.xml'))
        if 'bin' in options['coverage_format']:
            self.coverage.save()
        if 'html' in options['coverage_format']:
            # Dump coverage html
            self.coverage.html_report(morfs=morfs, directory=os.path.join(options['output_dir'], 'coverage'))

    def get_morfs(self, coverage, tested_locations, options):
        excluded = []

        # Exclude by module
        modnames = options.get('coverage_excludes') or getattr(settings, 'COVERAGE_EXCLUDES', [])
        for modname in modnames:
            try:
                excluded.append(os.path.dirname(import_module(modname).__file__))
            except ImportError:
                pass

        # Exclude by directory
        excluded.extend(getattr(settings, 'COVERAGE_EXCLUDES_FOLDERS', []))

        return [filename for filename in coverage.data.measured_files()
                if not (os.sep + 'migrations' + os.sep) in filename
                if not (os.sep + 'south_migrations' + os.sep) in filename
                if any(filename.startswith(location) for location in tested_locations)
                if not any(filename.startswith(location) for location in excluded)]

    def default_coverage_config(self):
        rcfile = getattr(settings, 'COVERAGE_RCFILE', 'coverage.rc')
        if os.path.exists(rcfile):
            return rcfile
        return None



import os

PROJECT_ROOT = os.path.abspath(os.path.dirname(__file__))

DEBUG = True
TEMPLATE_DEBUG = DEBUG
ROOT_URLCONF = 'test_app.urls'
SECRET_KEY = 'nokey'
MIDDLEWARE_CLASSES = ()

TEMPLATE_LOADERS = (
    'django.template.loaders.filesystem.Loader',
    'django.template.loaders.app_directories.Loader',
)

TEMPLATES = [
    {
        'BACKEND': 'django.template.backends.django.DjangoTemplates',
        'DIRS': [
        ],
        'APP_DIRS': True,
        'OPTIONS': {
            'context_processors': [
                'django.contrib.auth.context_processors.auth',
                'django.template.context_processors.debug',
                'django.template.context_processors.i18n',
                'django.template.context_processors.media',
                'django.template.context_processors.static',
                'django.template.context_processors.tz',
                'django.contrib.messages.context_processors.messages',
            ],
        },
    },
]

PROJECT_APPS = (
    'django.contrib.sessions',  # just to ensure that dotted apps test works
    'django_jenkins',
    'tests.test_app',
    'tests.test_app_dirs',
)

INSTALLED_APPS = (
    'django.contrib.auth',
    'django.contrib.contenttypes',
) + PROJECT_APPS


DATABASE_ENGINE = 'sqlite3'
DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.%s' % DATABASE_ENGINE,
        }
}

SOUTH_MIGRATION_MODULES = {
    'test_app': 'test_app.south_migrations',
}

JENKINS_TASKS = (
    'django_jenkins.tasks.run_pylint',
    'django_jenkins.tasks.run_pep8',
    'django_jenkins.tasks.run_csslint',
    'django_jenkins.tasks.run_flake8',
    'django_jenkins.tasks.run_pyflakes',
    'django_jenkins.tasks.run_jshint',
    'django_jenkins.tasks.run_sloccount',
)

COVERAGE_EXCLUDES = ['tests.test_app.not_for_coverage', ]
COVERAGE_EXCLUDES_FOLDERS = [os.path.join(PROJECT_ROOT, 'test_app_dirs/not_for_coverage/'), ]

# JSHINT_CHECKED_FILES = [os.path.join(PROJECT_ROOT, 'static/js/test.js')]
# CSSLINT_CHECKED_FILES = [os.path.join(PROJECT_ROOT, 'static/css/test.css')]

PYLINT_LOAD_PLUGIN = (
    'pylint_django',
)

STATICFILES_DIRS = [
    os.path.join(PROJECT_ROOT, 'static/'),
]

STATIC_URL = '/media/'


LOGGING = {
    'version': 1,
    'disable_existing_loggers': False,
    'handlers': {
        'console': {
            'level': 'DEBUG',
            'class': 'logging.StreamHandler',
        },
    },
    'loggers': {
        'django.request': {
            'handlers': ['console'],
            'level': 'ERROR',
            'propagate': True,
        },
    }
}



# -*- coding: utf-8 -*-
import io
import sys
from xml.etree import ElementTree as ET

from django.core import mail
from django.test import TestCase
from unittest import skip

from django.test import LiveServerTestCase
from pyvirtualdisplay import Display
from selenium import webdriver

from django_jenkins.runner import EXMLTestResult


class SaintyChecks(TestCase):
    # @classmethod
    # def setUpClass(cls):
    #    raise Exception("Ups, should be disabled")

    def test_mailbox_stubs_not_broken(self):
        print("Testing mailbox django stubs")
        mail.send_mail('Test subject', 'Test message', 'nobody@kenkins.com',
                       ['somewhere@nowhere.com'])
        self.assertTrue(1, len(mail.outbox))

    @skip("Check skiped test")
    def test_is_skipped(self):
        print("This test should be skipped")

    def test_junit_xml_with_utf8_stdout_and_stderr(self):
        sys.stdout.write('\xc4\x85')
        sys.stderr.write('\xc4\x85')

    def test_junit_xml_with_invalid_stdout_and_stderr_encoding(self):
        sys.stdout.write('\xc4')
        sys.stderr.write('\xc4')

    # def test_failure(self):
    #    raise Exception("Ups, should be disabled")


class EXMLTestResultTests(TestCase):
    def setUp(self):
        self.exml_result = EXMLTestResult(None, None, 1)
        self.exml_result.startTestRun()
        self.result_element = ET.SubElement(self.exml_result.tree, 'result')

    def test_non_ascii_traceback(self):
        try:
            self.explode_with_unicode_traceback()
        except ValueError:
            err = sys.exc_info()
        else:
            self.fail()

        self.exml_result._add_tb_to_test(TestCase('fail'), self.result_element, err)

        output = self.write_element(self.result_element)

        self.assert_(output)

    def test_non_ascii_message(self):
        try:
            self.explode_with_unicode_message()
        except ValueError:
            err = sys.exc_info()
        else:
            self.fail()

        self.exml_result._add_tb_to_test(TestCase('fail'), self.result_element, err)

        output = self.write_element(self.result_element)

        self.assert_(output)

    def write_element(self, element):
        # write out the element the way that our TestResult.dump_xml does.
        # (except not actually to disk.)
        tree = ET.ElementTree(element)
        output = io.BytesIO()
        # this bit blows up if components of the output are byte-strings with non-ascii content.
        tree.write(output, encoding='utf-8')
        output_bytes = output.getvalue()
        return output_bytes

    def explode_with_unicode_traceback(self):
        # The following will result in an ascii error message, but the traceback will contain the
        # full source line, including the comment's non-ascii characters.
        raise ValueError("dead")  # "⚠ Not enough ☕"

    def explode_with_unicode_message(self):
        # This source code has only ascii, the exception has a non-ascii message.
        raise ValueError(u"\N{BIOHAZARD SIGN} Too much \N{HOT BEVERAGE}")


class SeleniumTests(LiveServerTestCase):
    @classmethod
    def setUpClass(cls):
        cls.display = Display(visible=0, size=(1024, 768))
        cls.display.start()
        cls.selenium = webdriver.Firefox()
        super(SeleniumTests, cls).setUpClass()

    @classmethod
    def tearDownClass(cls):
        super(SeleniumTests, cls).tearDownClass()
        cls.selenium.quit()
        cls.display.stop()

    def test_login(self):
        self.selenium.get('%s%s' % (self.live_server_url, '/test_click/'))
        self.selenium.find_element_by_id("wm_click").click()
        self.assertEqual('Button clicked', self.selenium.find_element_by_id("wm_target").text)
